@@ -1,27 +1,33 @@
 import os
+import re
 from io import BytesIO
 
-from PIL import Image, ImageFile, JpegImagePlugin
+import pytest
+from PIL import (
+    ExifTags,
+    Image,
+    ImageFile,
+    JpegImagePlugin,
+    UnidentifiedImageError,
+    features,
+)
 
 from .helper import (
-    PillowTestCase,
+    assert_image,
+    assert_image_equal,
+    assert_image_similar,
     cjpeg_available,
     djpeg_available,
     hopper,
     is_win32,
-    unittest,
+    skip_unless_feature,
 )
 
-codecs = dir(Image.core)
-
 TEST_FILE = "Tests/images/hopper.jpg"
 
 
-class TestFileJpeg(PillowTestCase):
-    def setUp(self):
-        if "jpeg_encoder" not in codecs or "jpeg_decoder" not in codecs:
-            self.skipTest("jpeg support not available")
-
+@skip_unless_feature("jpg")
+class TestFileJpeg:
     def roundtrip(self, im, **options):
         out = BytesIO()
         im.save(out, "JPEG", **options)
@@ -42,25 +48,26 @@ class TestFileJpeg(PillowTestCase):
     def test_sanity(self):
 
         # internal version number
-        self.assertRegex(Image.core.jpeglib_version, r"\d+\.\d+$")
+        assert re.search(r"\d+\.\d+$", features.version_codec("jpg"))
 
         with Image.open(TEST_FILE) as im:
             im.load()
-            self.assertEqual(im.mode, "RGB")
-            self.assertEqual(im.size, (128, 128))
-            self.assertEqual(im.format, "JPEG")
-            self.assertEqual(im.get_format_mimetype(), "image/jpeg")
+            assert im.mode == "RGB"
+            assert im.size == (128, 128)
+            assert im.format == "JPEG"
+            assert im.get_format_mimetype() == "image/jpeg"
 
     def test_app(self):
         # Test APP/COM reader (@PIL135)
         with Image.open(TEST_FILE) as im:
-            self.assertEqual(
-                im.applist[0], ("APP0", b"JFIF\x00\x01\x01\x01\x00`\x00`\x00\x00")
+            assert im.applist[0] == ("APP0", b"JFIF\x00\x01\x01\x01\x00`\x00`\x00\x00")
+            assert im.applist[1] == (
+                "COM",
+                b"File written by Adobe Photoshop\xa8 4.0\x00",
             )
-            self.assertEqual(
-                im.applist[1], ("COM", b"File written by Adobe Photoshop\xa8 4.0\x00")
-            )
-            self.assertEqual(len(im.applist), 2)
+            assert len(im.applist) == 2
+
+            assert im.info["comment"] == b"File written by Adobe Photoshop\xa8 4.0\x00"
 
     def test_cmyk(self):
         # Test CMYK handling.  Thanks to Tim and Charlie for test data,
@@ -69,54 +76,57 @@ class TestFileJpeg(PillowTestCase):
         with Image.open(f) as im:
             # the source image has red pixels in the upper left corner.
             c, m, y, k = [x / 255.0 for x in im.getpixel((0, 0))]
-            self.assertEqual(c, 0.0)
-            self.assertGreater(m, 0.8)
-            self.assertGreater(y, 0.8)
-            self.assertEqual(k, 0.0)
+            assert c == 0.0
+            assert m > 0.8
+            assert y > 0.8
+            assert k == 0.0
             # the opposite corner is black
             c, m, y, k = [
                 x / 255.0 for x in im.getpixel((im.size[0] - 1, im.size[1] - 1))
             ]
-            self.assertGreater(k, 0.9)
+            assert k > 0.9
             # roundtrip, and check again
             im = self.roundtrip(im)
             c, m, y, k = [x / 255.0 for x in im.getpixel((0, 0))]
-            self.assertEqual(c, 0.0)
-            self.assertGreater(m, 0.8)
-            self.assertGreater(y, 0.8)
-            self.assertEqual(k, 0.0)
+            assert c == 0.0
+            assert m > 0.8
+            assert y > 0.8
+            assert k == 0.0
             c, m, y, k = [
                 x / 255.0 for x in im.getpixel((im.size[0] - 1, im.size[1] - 1))
             ]
-            self.assertGreater(k, 0.9)
+            assert k > 0.9
 
-    def test_dpi(self):
+    @pytest.mark.parametrize(
+        "test_image_path", [TEST_FILE, "Tests/images/pil_sample_cmyk.jpg"],
+    )
+    def test_dpi(self, test_image_path):
         def test(xdpi, ydpi=None):
-            with Image.open(TEST_FILE) as im:
+            with Image.open(test_image_path) as im:
                 im = self.roundtrip(im, dpi=(xdpi, ydpi or xdpi))
             return im.info.get("dpi")
 
-        self.assertEqual(test(72), (72, 72))
-        self.assertEqual(test(300), (300, 300))
-        self.assertEqual(test(100, 200), (100, 200))
-        self.assertIsNone(test(0))  # square pixels
+        assert test(72) == (72, 72)
+        assert test(300) == (300, 300)
+        assert test(100, 200) == (100, 200)
+        assert test(0) is None  # square pixels
 
-    def test_icc(self):
+    def test_icc(self, tmp_path):
         # Test ICC support
         with Image.open("Tests/images/rgb.jpg") as im1:
             icc_profile = im1.info["icc_profile"]
-            self.assertEqual(len(icc_profile), 3144)
+            assert len(icc_profile) == 3144
             # Roundtrip via physical file.
-            f = self.tempfile("temp.jpg")
+            f = str(tmp_path / "temp.jpg")
             im1.save(f, icc_profile=icc_profile)
         with Image.open(f) as im2:
-            self.assertEqual(im2.info.get("icc_profile"), icc_profile)
+            assert im2.info.get("icc_profile") == icc_profile
             # Roundtrip via memory buffer.
             im1 = self.roundtrip(hopper())
             im2 = self.roundtrip(hopper(), icc_profile=icc_profile)
-            self.assert_image_equal(im1, im2)
-            self.assertFalse(im1.info.get("icc_profile"))
-            self.assertTrue(im2.info.get("icc_profile"))
+            assert_image_equal(im1, im2)
+            assert not im1.info.get("icc_profile")
+            assert im2.info.get("icc_profile")
 
     def test_icc_big(self):
         # Make sure that the "extra" support handles large blocks
@@ -125,9 +135,9 @@ class TestFileJpeg(PillowTestCase):
             # using a 4-byte test code should allow us to detect out of
             # order issues.
             icc_profile = (b"Test" * int(n / 4 + 1))[:n]
-            self.assertEqual(len(icc_profile), n)  # sanity
+            assert len(icc_profile) == n  # sanity
             im1 = self.roundtrip(hopper(), icc_profile=icc_profile)
-            self.assertEqual(im1.info.get("icc_profile"), icc_profile or None)
+            assert im1.info.get("icc_profile") == (icc_profile or None)
 
         test(0)
         test(1)
@@ -140,14 +150,14 @@ class TestFileJpeg(PillowTestCase):
         test(ImageFile.MAXBLOCK + 1)  # full buffer block plus one byte
         test(ImageFile.MAXBLOCK * 4 + 3)  # large block
 
-    def test_large_icc_meta(self):
+    def test_large_icc_meta(self, tmp_path):
         # https://github.com/python-pillow/Pillow/issues/148
         # Sometimes the meta data on the icc_profile block is bigger than
         # Image.MAXBLOCK or the image size.
         with Image.open("Tests/images/icc_profile_big.jpg") as im:
-            f = self.tempfile("temp.jpg")
+            f = str(tmp_path / "temp.jpg")
             icc_profile = im.info["icc_profile"]
-            # Should not raise IOError for image with icc larger than image size.
+            # Should not raise OSError for image with icc larger than image size.
             im.save(
                 f,
                 format="JPEG",
@@ -161,14 +171,14 @@ class TestFileJpeg(PillowTestCase):
         im1 = self.roundtrip(hopper())
         im2 = self.roundtrip(hopper(), optimize=0)
         im3 = self.roundtrip(hopper(), optimize=1)
-        self.assert_image_equal(im1, im2)
-        self.assert_image_equal(im1, im3)
-        self.assertGreaterEqual(im1.bytes, im2.bytes)
-        self.assertGreaterEqual(im1.bytes, im3.bytes)
+        assert_image_equal(im1, im2)
+        assert_image_equal(im1, im3)
+        assert im1.bytes >= im2.bytes
+        assert im1.bytes >= im3.bytes
 
-    def test_optimize_large_buffer(self):
+    def test_optimize_large_buffer(self, tmp_path):
         # https://github.com/python-pillow/Pillow/issues/148
-        f = self.tempfile("temp.jpg")
+        f = str(tmp_path / "temp.jpg")
         # this requires ~ 1.5x Image.MAXBLOCK
         im = Image.new("RGB", (4096, 4096), 0xFF3333)
         im.save(f, format="JPEG", optimize=True)
@@ -177,21 +187,21 @@ class TestFileJpeg(PillowTestCase):
         im1 = self.roundtrip(hopper())
         im2 = self.roundtrip(hopper(), progressive=False)
         im3 = self.roundtrip(hopper(), progressive=True)
-        self.assertFalse(im1.info.get("progressive"))
-        self.assertFalse(im2.info.get("progressive"))
-        self.assertTrue(im3.info.get("progressive"))
+        assert not im1.info.get("progressive")
+        assert not im2.info.get("progressive")
+        assert im3.info.get("progressive")
 
-        self.assert_image_equal(im1, im3)
-        self.assertGreaterEqual(im1.bytes, im3.bytes)
+        assert_image_equal(im1, im3)
+        assert im1.bytes >= im3.bytes
 
-    def test_progressive_large_buffer(self):
-        f = self.tempfile("temp.jpg")
+    def test_progressive_large_buffer(self, tmp_path):
+        f = str(tmp_path / "temp.jpg")
         # this requires ~ 1.5x Image.MAXBLOCK
         im = Image.new("RGB", (4096, 4096), 0xFF3333)
         im.save(f, format="JPEG", progressive=True)
 
-    def test_progressive_large_buffer_highest_quality(self):
-        f = self.tempfile("temp.jpg")
+    def test_progressive_large_buffer_highest_quality(self, tmp_path):
+        f = str(tmp_path / "temp.jpg")
         im = self.gen_random_image((255, 255))
         # this requires more bytes than pixels in the image
         im.save(f, format="JPEG", progressive=True, quality=100)
@@ -202,9 +212,9 @@ class TestFileJpeg(PillowTestCase):
         im = self.gen_random_image((256, 256), "CMYK")
         im.save(f, format="JPEG", progressive=True, quality=94)
 
-    def test_large_exif(self):
+    def test_large_exif(self, tmp_path):
         # https://github.com/python-pillow/Pillow/issues/148
-        f = self.tempfile("temp.jpg")
+        f = str(tmp_path / "temp.jpg")
         im = hopper()
         im.save(f, "JPEG", quality=90, exif=b"1" * 65532)
 
@@ -219,7 +229,7 @@ class TestFileJpeg(PillowTestCase):
             gps_index = 34853
             expected_exif_gps = {
                 0: b"\x00\x00\x00\x01",
-                2: (4294967295, 1),
+                2: 4294967295,
                 5: b"\x01",
                 30: 65535,
                 29: "1999:99:99 99:99:99",
@@ -229,7 +239,7 @@ class TestFileJpeg(PillowTestCase):
             exif = im._getexif()
 
         # Assert
-        self.assertEqual(exif[gps_index], expected_exif_gps)
+        assert exif[gps_index] == expected_exif_gps
 
     def test_exif_rollback(self):
         # rolling back exif support in 3.1 to pre-3.0 formatting.
@@ -241,7 +251,7 @@ class TestFileJpeg(PillowTestCase):
             36867: "2099:09:29 10:10:10",
             34853: {
                 0: b"\x00\x00\x00\x01",
-                2: (4294967295, 1),
+                2: 4294967295,
                 5: b"\x01",
                 30: 65535,
                 29: "1999:99:99 99:99:99",
@@ -253,18 +263,18 @@ class TestFileJpeg(PillowTestCase):
             271: "Make",
             272: "XXX-XXX",
             305: "PIL",
-            42034: ((1, 1), (1, 1), (1, 1), (1, 1)),
+            42034: (1, 1, 1, 1),
             42035: "LensMake",
             34856: b"\xaa\xaa\xaa\xaa\xaa\xaa",
-            282: (4294967295, 1),
-            33434: (4294967295, 1),
+            282: 4294967295,
+            33434: 4294967295,
         }
 
         with Image.open("Tests/images/exif_gps.jpg") as im:
             exif = im._getexif()
 
         for tag, value in expected_exif.items():
-            self.assertEqual(value, exif[tag])
+            assert value == exif[tag]
 
     def test_exif_gps_typeerror(self):
         with Image.open("Tests/images/exif_gps_typeerror.jpg") as im:
@@ -274,35 +284,39 @@ class TestFileJpeg(PillowTestCase):
 
     def test_progressive_compat(self):
         im1 = self.roundtrip(hopper())
-        self.assertFalse(im1.info.get("progressive"))
-        self.assertFalse(im1.info.get("progression"))
+        assert not im1.info.get("progressive")
+        assert not im1.info.get("progression")
 
         im2 = self.roundtrip(hopper(), progressive=0)
         im3 = self.roundtrip(hopper(), progression=0)  # compatibility
-        self.assertFalse(im2.info.get("progressive"))
-        self.assertFalse(im2.info.get("progression"))
-        self.assertFalse(im3.info.get("progressive"))
-        self.assertFalse(im3.info.get("progression"))
+        assert not im2.info.get("progressive")
+        assert not im2.info.get("progression")
+        assert not im3.info.get("progressive")
+        assert not im3.info.get("progression")
 
         im2 = self.roundtrip(hopper(), progressive=1)
         im3 = self.roundtrip(hopper(), progression=1)  # compatibility
-        self.assert_image_equal(im1, im2)
-        self.assert_image_equal(im1, im3)
-        self.assertTrue(im2.info.get("progressive"))
-        self.assertTrue(im2.info.get("progression"))
-        self.assertTrue(im3.info.get("progressive"))
-        self.assertTrue(im3.info.get("progression"))
+        assert_image_equal(im1, im2)
+        assert_image_equal(im1, im3)
+        assert im2.info.get("progressive")
+        assert im2.info.get("progression")
+        assert im3.info.get("progressive")
+        assert im3.info.get("progression")
 
     def test_quality(self):
         im1 = self.roundtrip(hopper())
         im2 = self.roundtrip(hopper(), quality=50)
-        self.assert_image(im1, im2.mode, im2.size)
-        self.assertGreaterEqual(im1.bytes, im2.bytes)
+        assert_image(im1, im2.mode, im2.size)
+        assert im1.bytes >= im2.bytes
+
+        im3 = self.roundtrip(hopper(), quality=0)
+        assert_image(im1, im3.mode, im3.size)
+        assert im2.bytes > im3.bytes
 
     def test_smooth(self):
         im1 = self.roundtrip(hopper())
         im2 = self.roundtrip(hopper(), smooth=100)
-        self.assert_image(im1, im2.mode, im2.size)
+        assert_image(im1, im2.mode, im2.size)
 
     def test_subsampling(self):
         def getsampling(im):
@@ -311,48 +325,49 @@ class TestFileJpeg(PillowTestCase):
 
         # experimental API
         im = self.roundtrip(hopper(), subsampling=-1)  # default
-        self.assertEqual(getsampling(im), (2, 2, 1, 1, 1, 1))
+        assert getsampling(im) == (2, 2, 1, 1, 1, 1)
         im = self.roundtrip(hopper(), subsampling=0)  # 4:4:4
-        self.assertEqual(getsampling(im), (1, 1, 1, 1, 1, 1))
+        assert getsampling(im) == (1, 1, 1, 1, 1, 1)
         im = self.roundtrip(hopper(), subsampling=1)  # 4:2:2
-        self.assertEqual(getsampling(im), (2, 1, 1, 1, 1, 1))
+        assert getsampling(im) == (2, 1, 1, 1, 1, 1)
         im = self.roundtrip(hopper(), subsampling=2)  # 4:2:0
-        self.assertEqual(getsampling(im), (2, 2, 1, 1, 1, 1))
+        assert getsampling(im) == (2, 2, 1, 1, 1, 1)
         im = self.roundtrip(hopper(), subsampling=3)  # default (undefined)
-        self.assertEqual(getsampling(im), (2, 2, 1, 1, 1, 1))
+        assert getsampling(im) == (2, 2, 1, 1, 1, 1)
 
         im = self.roundtrip(hopper(), subsampling="4:4:4")
-        self.assertEqual(getsampling(im), (1, 1, 1, 1, 1, 1))
+        assert getsampling(im) == (1, 1, 1, 1, 1, 1)
         im = self.roundtrip(hopper(), subsampling="4:2:2")
-        self.assertEqual(getsampling(im), (2, 1, 1, 1, 1, 1))
+        assert getsampling(im) == (2, 1, 1, 1, 1, 1)
         im = self.roundtrip(hopper(), subsampling="4:2:0")
-        self.assertEqual(getsampling(im), (2, 2, 1, 1, 1, 1))
+        assert getsampling(im) == (2, 2, 1, 1, 1, 1)
         im = self.roundtrip(hopper(), subsampling="4:1:1")
-        self.assertEqual(getsampling(im), (2, 2, 1, 1, 1, 1))
+        assert getsampling(im) == (2, 2, 1, 1, 1, 1)
 
-        self.assertRaises(TypeError, self.roundtrip, hopper(), subsampling="1:1:1")
+        with pytest.raises(TypeError):
+            self.roundtrip(hopper(), subsampling="1:1:1")
 
     def test_exif(self):
         with Image.open("Tests/images/pil_sample_rgb.jpg") as im:
             info = im._getexif()
-            self.assertEqual(info[305], "Adobe Photoshop CS Macintosh")
+            assert info[305] == "Adobe Photoshop CS Macintosh"
 
     def test_mp(self):
         with Image.open("Tests/images/pil_sample_rgb.jpg") as im:
-            self.assertIsNone(im._getmp())
+            assert im._getmp() is None
 
-    def test_quality_keep(self):
+    def test_quality_keep(self, tmp_path):
         # RGB
         with Image.open("Tests/images/hopper.jpg") as im:
-            f = self.tempfile("temp.jpg")
+            f = str(tmp_path / "temp.jpg")
             im.save(f, quality="keep")
         # Grayscale
         with Image.open("Tests/images/hopper_gray.jpg") as im:
-            f = self.tempfile("temp.jpg")
+            f = str(tmp_path / "temp.jpg")
             im.save(f, quality="keep")
         # CMYK
         with Image.open("Tests/images/pil_sample_cmyk.jpg") as im:
-            f = self.tempfile("temp.jpg")
+            f = str(tmp_path / "temp.jpg")
             im.save(f, quality="keep")
 
     def test_junk_jpeg_header(self):
@@ -372,35 +387,35 @@ class TestFileJpeg(PillowTestCase):
         with Image.open(filename) as im:
             im.load()
             ImageFile.LOAD_TRUNCATED_IMAGES = False
-            self.assertIsNotNone(im.getbbox())
+            assert im.getbbox() is not None
 
-    def test_truncated_jpeg_throws_IOError(self):
+    def test_truncated_jpeg_throws_oserror(self):
         filename = "Tests/images/truncated_jpeg.jpg"
         with Image.open(filename) as im:
-            with self.assertRaises(IOError):
+            with pytest.raises(OSError):
                 im.load()
 
             # Test that the error is raised if loaded a second time
-            with self.assertRaises(IOError):
+            with pytest.raises(OSError):
                 im.load()
 
-    def _n_qtables_helper(self, n, test_file):
-        with Image.open(test_file) as im:
-            f = self.tempfile("temp.jpg")
-            im.save(f, qtables=[[n] * 64] * n)
-        with Image.open(f) as im:
-            self.assertEqual(len(im.quantization), n)
-            reloaded = self.roundtrip(im, qtables="keep")
-            self.assertEqual(im.quantization, reloaded.quantization)
+    def test_qtables(self, tmp_path):
+        def _n_qtables_helper(n, test_file):
+            with Image.open(test_file) as im:
+                f = str(tmp_path / "temp.jpg")
+                im.save(f, qtables=[[n] * 64] * n)
+            with Image.open(f) as im:
+                assert len(im.quantization) == n
+                reloaded = self.roundtrip(im, qtables="keep")
+                assert im.quantization == reloaded.quantization
 
-    def test_qtables(self):
         with Image.open("Tests/images/hopper.jpg") as im:
             qtables = im.quantization
             reloaded = self.roundtrip(im, qtables=qtables, subsampling=0)
-            self.assertEqual(im.quantization, reloaded.quantization)
-            self.assert_image_similar(im, self.roundtrip(im, qtables="web_low"), 30)
-            self.assert_image_similar(im, self.roundtrip(im, qtables="web_high"), 30)
-            self.assert_image_similar(im, self.roundtrip(im, qtables="keep"), 30)
+            assert im.quantization == reloaded.quantization
+            assert_image_similar(im, self.roundtrip(im, qtables="web_low"), 30)
+            assert_image_similar(im, self.roundtrip(im, qtables="web_high"), 30)
+            assert_image_similar(im, self.roundtrip(im, qtables="keep"), 30)
 
             # valid bounds for baseline qtable
             bounds_qtable = [int(s) for s in ("255 1 " * 32).split(None)]
@@ -439,7 +454,7 @@ class TestFileJpeg(PillowTestCase):
                 )
             ]
             # list of qtable lists
-            self.assert_image_similar(
+            assert_image_similar(
                 im,
                 self.roundtrip(
                     im, qtables=[standard_l_qtable, standard_chrominance_qtable]
@@ -448,7 +463,7 @@ class TestFileJpeg(PillowTestCase):
             )
 
             # tuple of qtable lists
-            self.assert_image_similar(
+            assert_image_similar(
                 im,
                 self.roundtrip(
                     im, qtables=(standard_l_qtable, standard_chrominance_qtable)
@@ -457,7 +472,7 @@ class TestFileJpeg(PillowTestCase):
             )
 
             # dict of qtable lists
-            self.assert_image_similar(
+            assert_image_similar(
                 im,
                 self.roundtrip(
                     im, qtables={0: standard_l_qtable, 1: standard_chrominance_qtable}
@@ -465,54 +480,57 @@ class TestFileJpeg(PillowTestCase):
                 30,
             )
 
-            self._n_qtables_helper(1, "Tests/images/hopper_gray.jpg")
-            self._n_qtables_helper(1, "Tests/images/pil_sample_rgb.jpg")
-            self._n_qtables_helper(2, "Tests/images/pil_sample_rgb.jpg")
-            self._n_qtables_helper(3, "Tests/images/pil_sample_rgb.jpg")
-            self._n_qtables_helper(1, "Tests/images/pil_sample_cmyk.jpg")
-            self._n_qtables_helper(2, "Tests/images/pil_sample_cmyk.jpg")
-            self._n_qtables_helper(3, "Tests/images/pil_sample_cmyk.jpg")
-            self._n_qtables_helper(4, "Tests/images/pil_sample_cmyk.jpg")
+            _n_qtables_helper(1, "Tests/images/hopper_gray.jpg")
+            _n_qtables_helper(1, "Tests/images/pil_sample_rgb.jpg")
+            _n_qtables_helper(2, "Tests/images/pil_sample_rgb.jpg")
+            _n_qtables_helper(3, "Tests/images/pil_sample_rgb.jpg")
+            _n_qtables_helper(1, "Tests/images/pil_sample_cmyk.jpg")
+            _n_qtables_helper(2, "Tests/images/pil_sample_cmyk.jpg")
+            _n_qtables_helper(3, "Tests/images/pil_sample_cmyk.jpg")
+            _n_qtables_helper(4, "Tests/images/pil_sample_cmyk.jpg")
 
             # not a sequence
-            self.assertRaises(ValueError, self.roundtrip, im, qtables="a")
+            with pytest.raises(ValueError):
+                self.roundtrip(im, qtables="a")
             # sequence wrong length
-            self.assertRaises(ValueError, self.roundtrip, im, qtables=[])
+            with pytest.raises(ValueError):
+                self.roundtrip(im, qtables=[])
             # sequence wrong length
-            self.assertRaises(ValueError, self.roundtrip, im, qtables=[1, 2, 3, 4, 5])
+            with pytest.raises(ValueError):
+                self.roundtrip(im, qtables=[1, 2, 3, 4, 5])
 
             # qtable entry not a sequence
-            self.assertRaises(ValueError, self.roundtrip, im, qtables=[1])
+            with pytest.raises(ValueError):
+                self.roundtrip(im, qtables=[1])
             # qtable entry has wrong number of items
-            self.assertRaises(ValueError, self.roundtrip, im, qtables=[[1, 2, 3, 4]])
+            with pytest.raises(ValueError):
+                self.roundtrip(im, qtables=[[1, 2, 3, 4]])
 
-    @unittest.skipUnless(djpeg_available(), "djpeg not available")
+    @pytest.mark.skipif(not djpeg_available(), reason="djpeg not available")
     def test_load_djpeg(self):
         with Image.open(TEST_FILE) as img:
             img.load_djpeg()
-            self.assert_image_similar(img, Image.open(TEST_FILE), 0)
+            assert_image_similar(img, Image.open(TEST_FILE), 5)
 
-    @unittest.skipUnless(cjpeg_available(), "cjpeg not available")
-    def test_save_cjpeg(self):
+    @pytest.mark.skipif(not cjpeg_available(), reason="cjpeg not available")
+    def test_save_cjpeg(self, tmp_path):
         with Image.open(TEST_FILE) as img:
-            tempfile = self.tempfile("temp.jpg")
+            tempfile = str(tmp_path / "temp.jpg")
             JpegImagePlugin._save_cjpeg(img, 0, tempfile)
             # Default save quality is 75%, so a tiny bit of difference is alright
-            self.assert_image_similar(img, Image.open(tempfile), 17)
+            assert_image_similar(img, Image.open(tempfile), 17)
 
     def test_no_duplicate_0x1001_tag(self):
         # Arrange
-        from PIL import ExifTags
-
         tag_ids = {v: k for k, v in ExifTags.TAGS.items()}
 
         # Assert
-        self.assertEqual(tag_ids["RelatedImageWidth"], 0x1001)
-        self.assertEqual(tag_ids["RelatedImageLength"], 0x1002)
+        assert tag_ids["RelatedImageWidth"] == 0x1001
+        assert tag_ids["RelatedImageLength"] == 0x1002
 
-    def test_MAXBLOCK_scaling(self):
+    def test_MAXBLOCK_scaling(self, tmp_path):
         im = self.gen_random_image((512, 512))
-        f = self.tempfile("temp.jpeg")
+        f = str(tmp_path / "temp.jpeg")
         im.save(f, quality=100, optimize=True)
 
         with Image.open(f) as reloaded:
@@ -528,10 +546,10 @@ class TestFileJpeg(PillowTestCase):
         # Act
         # Shouldn't raise error
         fn = "Tests/images/sugarshack_bad_mpo_header.jpg"
-        with self.assert_warning(UserWarning, Image.open, fn) as im:
+        with pytest.warns(UserWarning, Image.open, fn) as im:
 
             # Assert
-            self.assertEqual(im.format, "JPEG")
+            assert im.format == "JPEG"
 
     def test_save_correct_modes(self):
         out = BytesIO()
@@ -544,11 +562,12 @@ class TestFileJpeg(PillowTestCase):
         out = BytesIO()
         for mode in ["LA", "La", "RGBA", "RGBa", "P"]:
             img = Image.new(mode, (20, 20))
-            self.assertRaises(IOError, img.save, out, "JPEG")
+            with pytest.raises(OSError):
+                img.save(out, "JPEG")
 
-    def test_save_tiff_with_dpi(self):
+    def test_save_tiff_with_dpi(self, tmp_path):
         # Arrange
-        outfile = self.tempfile("temp.tif")
+        outfile = str(tmp_path / "temp.tif")
         with Image.open("Tests/images/hopper.tif") as im:
 
             # Act
@@ -557,29 +576,29 @@ class TestFileJpeg(PillowTestCase):
             # Assert
             with Image.open(outfile) as reloaded:
                 reloaded.load()
-                self.assertEqual(im.info["dpi"], reloaded.info["dpi"])
+                assert im.info["dpi"] == reloaded.info["dpi"]
 
     def test_load_dpi_rounding(self):
         # Round up
         with Image.open("Tests/images/iptc_roundUp.jpg") as im:
-            self.assertEqual(im.info["dpi"], (44, 44))
+            assert im.info["dpi"] == (44, 44)
 
         # Round down
         with Image.open("Tests/images/iptc_roundDown.jpg") as im:
-            self.assertEqual(im.info["dpi"], (2, 2))
+            assert im.info["dpi"] == (2, 2)
 
-    def test_save_dpi_rounding(self):
-        outfile = self.tempfile("temp.jpg")
+    def test_save_dpi_rounding(self, tmp_path):
+        outfile = str(tmp_path / "temp.jpg")
         with Image.open("Tests/images/hopper.jpg") as im:
             im.save(outfile, dpi=(72.2, 72.2))
 
             with Image.open(outfile) as reloaded:
-                self.assertEqual(reloaded.info["dpi"], (72, 72))
+                assert reloaded.info["dpi"] == (72, 72)
 
             im.save(outfile, dpi=(72.8, 72.8))
 
         with Image.open(outfile) as reloaded:
-            self.assertEqual(reloaded.info["dpi"], (73, 73))
+            assert reloaded.info["dpi"] == (73, 73)
 
     def test_dpi_tuple_from_exif(self):
         # Arrange
@@ -588,7 +607,7 @@ class TestFileJpeg(PillowTestCase):
         with Image.open("Tests/images/photoshop-200dpi.jpg") as im:
 
             # Act / Assert
-            self.assertEqual(im.info.get("dpi"), (200, 200))
+            assert im.info.get("dpi") == (200, 200)
 
     def test_dpi_int_from_exif(self):
         # Arrange
@@ -597,7 +616,7 @@ class TestFileJpeg(PillowTestCase):
         with Image.open("Tests/images/exif-72dpi-int.jpg") as im:
 
             # Act / Assert
-            self.assertEqual(im.info.get("dpi"), (72, 72))
+            assert im.info.get("dpi") == (72, 72)
 
     def test_dpi_from_dpcm_exif(self):
         # Arrange
@@ -606,7 +625,7 @@ class TestFileJpeg(PillowTestCase):
         with Image.open("Tests/images/exif-200dpcm.jpg") as im:
 
             # Act / Assert
-            self.assertEqual(im.info.get("dpi"), (508, 508))
+            assert im.info.get("dpi") == (508, 508)
 
     def test_dpi_exif_zero_division(self):
         # Arrange
@@ -616,7 +635,7 @@ class TestFileJpeg(PillowTestCase):
 
             # Act / Assert
             # This should return the default, and not raise a ZeroDivisionError
-            self.assertEqual(im.info.get("dpi"), (72, 72))
+            assert im.info.get("dpi") == (72, 72)
 
     def test_no_dpi_in_exif(self):
         # Arrange
@@ -627,7 +646,7 @@ class TestFileJpeg(PillowTestCase):
             # Act / Assert
             # "When the image resolution is unknown, 72 [dpi] is designated."
             # http://www.exiv2.org/tags.html
-            self.assertEqual(im.info.get("dpi"), (72, 72))
+            assert im.info.get("dpi") == (72, 72)
 
     def test_invalid_exif(self):
         # This is no-dpi-in-exif with the tiff header of the exif block
@@ -636,15 +655,28 @@ class TestFileJpeg(PillowTestCase):
 
             # This should return the default, and not a SyntaxError or
             # OSError for unidentified image.
-            self.assertEqual(im.info.get("dpi"), (72, 72))
+            assert im.info.get("dpi") == (72, 72)
+
+    def test_exif_x_resolution(self, tmp_path):
+        with Image.open("Tests/images/flower.jpg") as im:
+            exif = im.getexif()
+            assert exif[282] == 180
+
+            out = str(tmp_path / "out.jpg")
+            with pytest.warns(None) as record:
+                im.save(out, exif=exif)
+            assert len(record) == 0
+
+        with Image.open(out) as reloaded:
+            assert reloaded.getexif()[282] == 180
 
     def test_invalid_exif_x_resolution(self):
         # When no x or y resolution is defined in EXIF
-        im = Image.open("Tests/images/invalid-exif-without-x-resolution.jpg")
+        with Image.open("Tests/images/invalid-exif-without-x-resolution.jpg") as im:
 
-        # This should return the default, and not a ValueError or
-        # OSError for an unidentified image.
-        self.assertEqual(im.info.get("dpi"), (72, 72))
+            # This should return the default, and not a ValueError or
+            # OSError for an unidentified image.
+            assert im.info.get("dpi") == (72, 72)
 
     def test_ifd_offset_exif(self):
         # Arrange
@@ -653,47 +685,71 @@ class TestFileJpeg(PillowTestCase):
         with Image.open("Tests/images/exif-ifd-offset.jpg") as im:
 
             # Act / Assert
-            self.assertEqual(im._getexif()[306], "2017:03:13 23:03:09")
+            assert im._getexif()[306] == "2017:03:13 23:03:09"
 
     def test_photoshop(self):
         with Image.open("Tests/images/photoshop-200dpi.jpg") as im:
-            self.assertEqual(
-                im.info["photoshop"][0x03ED],
-                {
-                    "XResolution": 200.0,
-                    "DisplayedUnitsX": 1,
-                    "YResolution": 200.0,
-                    "DisplayedUnitsY": 1,
-                },
-            )
+            assert im.info["photoshop"][0x03ED] == {
+                "XResolution": 200.0,
+                "DisplayedUnitsX": 1,
+                "YResolution": 200.0,
+                "DisplayedUnitsY": 1,
+            }
 
             # Test that the image can still load, even with broken Photoshop data
             # This image had the APP13 length hexedited to be smaller
             with Image.open("Tests/images/photoshop-200dpi-broken.jpg") as im_broken:
-                self.assert_image_equal(im_broken, im)
+                assert_image_equal(im_broken, im)
 
         # This image does not contain a Photoshop header string
         with Image.open("Tests/images/app13.jpg") as im:
-            self.assertNotIn("photoshop", im.info)
+            assert "photoshop" not in im.info
+
+    def test_photoshop_malformed_and_multiple(self):
+        with Image.open("Tests/images/app13-multiple.jpg") as im:
+            assert "photoshop" in im.info
+            assert 24 == len(im.info["photoshop"])
+            apps_13_lengths = [len(v) for k, v in im.applist if k == "APP13"]
+            assert [65504, 24] == apps_13_lengths
+
+    def test_icc_after_SOF(self):
+        with Image.open("Tests/images/icc-after-SOF.jpg") as im:
+            assert im.info["icc_profile"] == b"profile"
+
+    def test_jpeg_magic_number(self):
+        size = 4097
+        buffer = BytesIO(b"\xFF" * size)  # Many xFF bytes
+        buffer.max_pos = 0
+        orig_read = buffer.read
+
+        def read(n=-1):
+            res = orig_read(n)
+            buffer.max_pos = max(buffer.max_pos, buffer.tell())
+            return res
+
+        buffer.read = read
+        with pytest.raises(UnidentifiedImageError):
+            Image.open(buffer)
+
+        # Assert the entire file has not been read
+        assert 0 < buffer.max_pos < size
 
 
-@unittest.skipUnless(is_win32(), "Windows only")
-class TestFileCloseW32(PillowTestCase):
-    def setUp(self):
-        if "jpeg_encoder" not in codecs or "jpeg_decoder" not in codecs:
-            self.skipTest("jpeg support not available")
-
-    def test_fd_leak(self):
-        tmpfile = self.tempfile("temp.jpg")
+@pytest.mark.skipif(not is_win32(), reason="Windows only")
+@skip_unless_feature("jpg")
+class TestFileCloseW32:
+    def test_fd_leak(self, tmp_path):
+        tmpfile = str(tmp_path / "temp.jpg")
 
         with Image.open("Tests/images/hopper.jpg") as im:
             im.save(tmpfile)
 
         im = Image.open(tmpfile)
         fp = im.fp
-        self.assertFalse(fp.closed)
-        self.assertRaises(WindowsError, os.remove, tmpfile)
+        assert not fp.closed
+        with pytest.raises(OSError):
+            os.remove(tmpfile)
         im.load()
-        self.assertTrue(fp.closed)
+        assert fp.closed
         # this should not fail, as load should have closed the file.
         os.remove(tmpfile)
